/*
 * $Id: JarInstrumenter.java,v 1.3 2005/10/25 11:04:09 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 09.06.2005 10:32:00 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.bcel.instrumenter;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.zip.ZipFile;

import org.apache.bcel.Repository;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.util.ClassLoaderRepository;
import org.deltadebugging.ddstate.launcher.server.instrumenter.CreateInstanceInstrumenter;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.softevo.sibrelib.source.ClassFile;
import org.softevo.sibrelib.source.JarSource;

/**
 * @author lehmann
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class JarInstrumenter {

    public static void createModifiedJar(String zipFileName,
            String modPathName, String modFileName,
            IClassInstrumenter classInstrumenter, IProgressMonitor monitor)
            throws ModifiedJarCreationException {

        if (monitor == null)
            monitor = new NullProgressMonitor();

        // look if the path exists
        File targetPath = new File(modPathName);
        if (!targetPath.exists()) {
            targetPath.mkdirs();
        }

        File targetFile = new File(modPathName + File.separator + modFileName);

        try {
            monitor.beginTask("Create modified runtime jar...", 100);

            //
            // load the runtime jar for the modifications
            // 
            JarSource source = null;

            if (!monitor.isCanceled()) {
                monitor.subTask("Load runtime jar...");

                source = new JarSource(new ZipFile(zipFileName), targetFile);
                Repository.clearCache();
                Repository.setRepository(new ClassLoaderRepository(source
                        .createBCELClassLoader()));

                monitor.worked(10);
            }

            //
            // modify all classes in the source
            //
            if (!monitor.isCanceled()) {
                modifyClasses(source, classInstrumenter,
                        new SubProgressMonitor(monitor, 80));
            }

            //
            // store the modified classes
            //
            if (!monitor.isCanceled()) {
                monitor.subTask("Save modified runtime jar...");                
                source.saveModifiedVersion();
                
                monitor.worked(10);
            }

        } catch (Throwable t) {
            targetFile.delete();
            throw new ModifiedJarCreationException(
                    "Cannot create modified jar", t);

        } finally {
            monitor.done();

        }

    }

    private static void modifyClasses(JarSource jarSource,
            IClassInstrumenter classInstrumenter, IProgressMonitor monitor)
            throws ClassNotFoundException, InstrumentationException {

        try {
            monitor.beginTask("Modify classes", jarSource.getSize());
            
            Iterator itClasses = jarSource.getClassIterator();

            while (itClasses.hasNext()) {
                ClassFile clazz = (ClassFile) itClasses.next();
                monitor.subTask("Modify " + clazz.getClassName() + "...");

                // load the class
                JavaClass javaClass = loadClass(clazz.getClassName());
                javaClass = classInstrumenter.modifyClass(javaClass);

                ByteArrayOutputStream byteBuffer = new ByteArrayOutputStream();
                try {
                    javaClass.dump(byteBuffer);

                } catch (IOException exception) {
                    //this exception is never raised when we use
                    //a ByteArrayOutputStream
                    throw new ClassNotFoundException(
                            "An exception occured dumping a java class: "
                                    + exception);
                }
                clazz.setModifiedClassBytes(byteBuffer.toByteArray());
                monitor.worked(1);
            }

        } finally {
            monitor.done();

        }
    }

    public static void main(String[] args) throws ModifiedJarCreationException {
        IClassInstrumenter classIntrumenter = new CreateInstanceInstrumenter();

        JarInstrumenter
                .createModifiedJar(
                        "C:/Programme/Development/j2sdk1.4.2_07/jre/lib/rt.jar",
                        "D:/Studium/Diplom/development/testprojects/StateTransitionTest/testJars",
                        "mod_rt.jar", classIntrumenter, null);

        JarInstrumenter
                .createModifiedJar(
                        "D:/Studium/Diplom/development/testprojects/StateTransitionTest/testJars/log4j-1.2.9.jar",
                        "D:/Studium/Diplom/development/testprojects/StateTransitionTest/testJars",
                        "mod_log4j-1.2.9.jar", classIntrumenter, null);

        JarInstrumenter
                .createModifiedJar(
                        "D:/Studium/Diplom/development/testprojects/StateTransitionTest/testJars/state.jar",
                        "D:/Studium/Diplom/development/testprojects/StateTransitionTest/testJars",
                        "mod_state.jar", classIntrumenter, null);
    }

    /**
     * This method loads a class using bcel. <BR>
     * 
     * @param name
     *            the name of the class that is loaded
     * @return the bcel class corresponding to the name
     * @throws ClassNotFoundException
     *             if the class could not be loaded
     */
    public static JavaClass loadClass(String name)
            throws ClassNotFoundException {
        JavaClass result;

        result = Repository.lookupClass(name);
        if (result == null) {
            throw new ClassNotFoundException(name);
        }

        //else
        return result;
    }
}