/*
 * $Id: SimpleDDLauncher.java,v 1.25 2005/10/25 11:09:49 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.01.2005 18:00:59 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client;

import org.deltadebugging.ddstate.comparator.exceptions.GraphComparatorException;
import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.client.exception.DeltaComputationException;
import org.deltadebugging.ddstate.launcher.ddrun.DDRunDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPoint;
import org.deltadebugging.ddstate.launcher.matcher.SimpleMethodCallMatcher;
import org.deltadebugging.ddstate.launcher.matcher.exception.MatchPointComputationException;
import org.deltadebugging.ddstate.launcher.tester.exception.TesterException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;

/**
 * COMMENTME Comment type SimpleDDLauncher
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public class SimpleDDLauncher extends BaseLauncher {

    private static final double[] SEARCH_POINTS = new double[] { 0.2, 0.65, 0.8 };

    public SimpleDDLauncher() {
        super(new SimpleMethodCallMatcher(SEARCH_POINTS));
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.launcher.client.BaseLauncher#startLauncherStrategy(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void startLauncherStrategy(IProgressMonitor monitor) {

        IMatchPoint[] matchPoints = null;

        try {
            monitor.beginTask("Start SimpleDDLaucnher", 100);

            //
            // if the match point finder is set and enabled, start it and find
            // the comparable breakpoint in the test runs
            //
            if (!monitor.isCanceled()) {
                if (this.getMatchPointFinder() != null
                        && this.isMatchPointFinderEnabled()) {

                    try {
                        matchPoints = this.getMatchPointFinder()
                                .computeMatchPoints(
                                        this.getLaunchDescription(),
                                        new SubProgressMonitor(monitor, 10));

                    } catch (MatchPointComputationException e) {
                        LaunchExceptionHandler.handleException(e);
                        monitor.setCanceled(true);
                    }

                } else {                    
                    // read the match points from the launch descriptor
                    matchPoints = this.getLaunchDescription().getMatchPoints();
                    
                    monitor.worked(10);

                }
            }

            if (!monitor.isCanceled()) {
                if (matchPoints == null || matchPoints.length == 0) {
                    // inform the use if no matchpoints could computed
                    LaunchExceptionHandler
                            .handleInformation("The automatically computation of the match points produce no result. "
                                    + "Please use the match point tab of your launch configuration "
                                    + "to determine the match point by hand.");
                    monitor.setCanceled(true);
                }
            }

            //
            // start deltadebugging for all match points in the dd run
            // description
            //
            if (!monitor.isCanceled()) {
                int totalTicks = 90;
                int ticksPerMP = totalTicks / matchPoints.length;
                int usedTicks = -1;
                
                for (int i = 0; i < matchPoints.length; i++) {                    
                    IDDRunDescription ddRunDesc = new DDRunDescription(this
                            .getLaunchDescription(), matchPoints[i]);
                                                           
                    if (i < (matchPoints.length - 1)) {
                        usedTicks = ticksPerMP;                        
                    } else {
                        usedTicks = totalTicks;
                    }
                    totalTicks -= usedTicks;
                    
                    IDDRunResult ddRunResult = this.computeRelevantDeltas(
                            ddRunDesc, new SubProgressMonitor(monitor, usedTicks));

                    // 
                    // FIXME intdroce a new listener concept. Move this
                    // notification to BaseLauncher
                    // notify all dd run listeners that the delta debugging run
                    // is terminated
                    //
                    DDStateLaunchPlugin.getDDRunListenerManager()
                            .notifyDDRunTerminated(ddRunResult);
                }
            }

        } catch (GraphComparatorException e) {
            LaunchExceptionHandler.handleException(e);

        } catch (TestRunnerException e) {
            LaunchExceptionHandler.handleException(e);

        } catch (DeltaComputationException e) {
            LaunchExceptionHandler.handleException(e);

        } catch (TesterException e) {
            LaunchExceptionHandler.handleException(e);

        } finally {
            monitor.done();
        }

    }

}