/*
 * $Id: StackTrace.java,v 1.3 2005/07/26 13:26:03 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Jul 21, 2005 12:22:05 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.matcher;

import java.util.Arrays;

import org.deltadebugging.ddstate.launcher.server.instrumenter.IdentifierMap;
import org.deltadebugging.ddstate.launcher.server.instrumenter.MethodIdentifier;

/**
 * COMMENTME Comment type StackTrace
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class StackTrace {

    private final int[] fTrace;
    private final int[] fCountTrace;
    private final IdentifierMap fIdentifierMap;

    public StackTrace(IdentifierMap identifierMap) {
        this(new int[0], new int[0], identifierMap);
    }

    public StackTrace(int methodID, int count, IdentifierMap identifierMap) {
        this(new int[] { methodID }, new int[] { count }, identifierMap);
    }

    public StackTrace(int[] methods, int[] countTrace,
            IdentifierMap identifierMap) {
        this.fTrace = methods;
        this.fCountTrace = countTrace;
        this.fIdentifierMap = identifierMap;
    }

    public StackTrace pushMethod(int methodID, int count) {
        StackTrace result = null;

        int[] newTrace = new int[this.fTrace.length + 1];
        int[] newCountTrace = new int[this.fCountTrace.length + 1];

        System.arraycopy(this.fTrace, 0, newTrace, 0, this.fTrace.length);
        System.arraycopy(this.fCountTrace, 0, newCountTrace, 0,
                this.fCountTrace.length);

        newTrace[this.fTrace.length] = methodID;
        newCountTrace[this.fCountTrace.length] = count;

        result = new StackTrace(newTrace, newCountTrace, this.fIdentifierMap);

        return result;
    }

    public StackTrace popMethod() {
        StackTrace result = null;

        if (this.length() <= 0)
            throw new IllegalStateException(
                    "The stacktrace has no method to pop");

        int[] newTrace = new int[this.fTrace.length - 1];
        int[] newCountTrace = new int[this.fCountTrace.length - 1];

        System.arraycopy(this.fTrace, 0, newTrace, 0, this.fTrace.length - 1);
        System.arraycopy(this.fCountTrace, 0, newCountTrace, 0,
                this.fCountTrace.length - 1);

        result = new StackTrace(newTrace, newCountTrace, this.fIdentifierMap);

        return result;
    }

    public int[] getMethodIDTrace() {
        int[] result = new int[this.fTrace.length];

        System.arraycopy(this.fTrace, 0, result, 0, this.fTrace.length);

        return result;
    }

    public boolean equals(Object obj) {
        boolean result = false;

        if (obj instanceof StackTrace) {
            StackTrace traceToCompare = (StackTrace) obj;
            result = Arrays.equals(this.fTrace, traceToCompare
                    .getMethodIDTrace());
        }

        return result;
    }

    public int length() {
        return this.fTrace.length;
    }

    public MethodIdentifier getMethodOnTop() {
        MethodIdentifier result = null;

        if (this.length() > 0) {
            int methodID = this.fTrace[this.length() - 1];
            result = this.fIdentifierMap.getMethodIdentifier(methodID);
        }

        return result;
    }

    public int getCountOnTop() {
        int result = 0;

        if (this.length() > 0) {
            result = this.fCountTrace[this.length() - 1];
        }

        return result;
    }

    public String toString() {
        StringBuffer result = new StringBuffer();

        for (int i = 0; i < this.fTrace.length; i++) {
            int methodID = this.fTrace[i];

            MethodIdentifier methodIdentifier = null;
            methodIdentifier = this.fIdentifierMap
                    .getMethodIdentifier(methodID);

            if (methodIdentifier != null) {
                result.append("Method enter: " + methodIdentifier.toString()
                        + " (# " + this.fCountTrace[i] + ")\n");
            } else {
                result
                        .append("Error: cannot found method identifier for methodID = "
                                + methodID);
            }
        }

        return result.toString();
    }
}
