/*
 * $Id: WorkspaceTest.java,v 1.5 2005/08/10 14:33:14 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 31.05.2005 18:04:28 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.test;

import junit.framework.Assert;

import org.deltadebugging.ddstate.launcher.client.IDDLauncher;
import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.client.SimpleDDLauncher;
import org.deltadebugging.ddstate.launcher.client.testrunner.JUnitTestDescription;
import org.deltadebugging.ddstate.launcher.ddrun.DDLaunchDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaProject;

/**
 * COMMENTME Comment type WorkspaceTest
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class WorkspaceTest extends GenericDDStateTest {

    public WorkspaceTest() {
        super("Workspace test");
    }

    public void testHallo() {
        System.out.println("Hallo");
    }

    public void testStartDDLauncher() {
        IDDLauncher launcher = new SimpleDDLauncher();

        try {
            launcher.start(null, null);
            Assert.fail();
        } catch (IllegalArgumentException e) {
            // do nothing
        }
    }

    public void testFindJavaProjects() {
        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        IWorkspaceRoot root = workspace.getRoot();

        Assert.assertTrue(root.getProjects().length > 0);
    }

    public void testFindTestProject() {
        IJavaProject project = getJavaProject();
        Assert.assertTrue(project.exists());
    }

    public void testFindVMInstall() {
        try {
            getVMInstall();
        } catch (CoreException e) {
            Assert.fail();
        }
    }

    public void testSimpleLaunchWithoutStopPoints() {
        IDDLauncher launcher = new SimpleDDLauncher();

        try {

            IJUnitTestDescription passJUnitTestDesc = new JUnitTestDescription(
                    "XYZ", "testPassingRun");
            IJUnitTestDescription failJUnitTestDesc = new JUnitTestDescription(
                    "XYZ", "testFailingRun");

            IDDLaunchDescription ddLaunchDesc = new DDLaunchDescription(this
                    .getJavaProject(), this.getVMInstall(), passJUnitTestDesc,
                    failJUnitTestDesc);

            launcher.start(ddLaunchDesc, null);
        } catch (CoreException e) {
            Assert.fail();
        }
    }
}
