/*
 * $Id: ComparisonPredicate.java,v 1.2 2005/09/13 11:05:42 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Sep 12, 2005 1:42:53 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.tester.predicate;

import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;

import junit.framework.TestFailure;

/**
 * The exception predicate defines a predicate for every failed junit test run,
 * which failed with an AssertionFailure caused by an assert on a value
 * comparation. </br>
 * 
 * e.g.: <code>Assert.assertEquals("foo", "bar")</code>
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public class ComparisonPredicate extends FailurePredicate {

	private static final long serialVersionUID = 531058278635360614L;

	/**
	 * The expected value.
	 */
	private final String fExpectedValue;

	/**
	 * The actual value.
	 */
	private final String fActualValue;

	public ComparisonPredicate(TestFailure failure,
			IJUnitTestDescription testDesc, String expectedValue,
			String actualValue) {
		super(failure, testDesc);
		this.fExpectedValue = expectedValue;
		this.fActualValue = actualValue;
	}

	/**
	 * Returns the expected value.
	 * 
	 * @return the expected value.
	 */
	public String getExpectedValue() {
		return this.fExpectedValue;
	}

	/**
	 * Returns the actual value.
	 * 
	 * @return the actual value.
	 */
	public String getActualValue() {
		return this.fActualValue;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.tester.predicate.IPredicate#matches(org.deltadebugging.ddstate.launcher.tester.predicate.IPredicate)
	 */
	public boolean matches(IPredicate predicate) {
		boolean result = false;

		if (predicate instanceof ComparisonPredicate) {
			result = this.matches((ComparisonPredicate) predicate);
		}

		return result;
	}

	/**
	 * Two predicates matches if they define actual value of the two predicates
	 * are equal.
	 * 
	 * @param predicate
	 * @return true if the two predicates matches, otherwise false.
	 */
	public boolean matches(ComparisonPredicate predicate) {
		boolean result = super.matches(predicate);

		result &= this.getActualValue().equals(predicate.getActualValue());

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("ComparisonPredicate\n");
		result.append("\texpected value   : " + this.fExpectedValue + "\n");
		result.append("\tactual value     : " + this.fActualValue + "\n");
		result.append("\texception message: " + this.getExceptionMessage()
				+ "\n");
		result.append("\texception trace  : "
				+ this.getExceptionTraceAsString());

		return result.toString();
	}

}
