/*
 * $Id: IPredicate.java,v 1.2 2005/09/13 11:06:07 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Sep 12, 2005 10:27:03 AM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.tester.predicate;

import java.io.Serializable;

/**
 * The predicate defines the outcome of one junit test run.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public interface IPredicate extends Serializable {

	/**
	 * The test run ends without any assertion failure or errors.
	 */
	public static final int STATE_TEST_SUCCESSFUL = 1;

	/**
	 * The test run aborts with an assertion failure.
	 */
	public static final int STATE_TEST_WITH_FAILURES = 2;

	/**
	 * The test run aborts with an error.
	 */
	public static final int STATE_TEST_WITH_ERRORS = 4;

	/**
	 * Returns true if the given predicate is matches this predicate, otherwise
	 * false.
	 * 
	 * @param predicate
	 *            the predicate to compare.
	 * @return Returns true if the given predicate is equal to this predicate,
	 *         otherwise false.
	 */
	public boolean matches(IPredicate predicate);

	/**
	 * Returns the state of the predicate. This can be
	 * <code>STATE_TEST_SUCCESSFUL</code>,
	 * <code>STATE_TEST_WITH_FAILURES</code> or
	 * <code>STATE_TEST_WITH_ERRORS</code>.
	 * 
	 * @return the state of the predicate.
	 */
	public int getState();

	/**
	 * Returns true if the state of the predicate is
	 * <code>STATE_TEST_SUCCESSFUL</code>, otherwise false.
	 * 
	 * @return true if the state of the predicate is
	 *         <code>STATE_TEST_SUCCESSFUL</code>, otherwise false.
	 */
	public boolean wasSuccessful();
}
