/*
 * $Id: DDStateLaunchImages.java,v 1.1 2006/01/31 18:57:02 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 09.04.2005 18:32:14 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ui.tabgroup;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;

/**
 * COMMENTME Comment type DDStateLaunchImages
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDStateLaunchImages {

    // Personal logger for this class
    final static DDLogger logger = DDStateLaunchPlugin
            .getLogger(DDStateLaunchImages.class);

    public static String IMG_KEY_TEST = "Test";
    public static String IMG_KEY_MATCHPOINT = "Matchpoint";
    
    /**
     * The image registry containing <code>Image</code>s.
     */
    private static ImageRegistry imageRegistry;

    /**
     * A table of all the <code>ImageDescriptor</code>s.
     */
    private static Map imageDescriptors;

    /* Declare Common paths */
    private static URL ICON_BASE_URL = null;

    static {
        String pathSuffix = "icons/"; //$NON-NLS-1$
        ICON_BASE_URL = DDStateLaunchPlugin.getDefault().getBundle().getEntry(
                pathSuffix);
    }

    // Use IPath and toOSString to build the names to ensure they have the
    // slashes correct

    // enabled - size 16x16
    private final static String ELCL = "enabled/"; //$NON-NLS-1$

    /**
     * Declare all images
     */
    private static void declareImages() {
        // enabled toolbars
        declareRegistryImage(IMG_KEY_TEST, ELCL
                + "test.gif"); //$NON-NLS-1$
        declareRegistryImage(IMG_KEY_MATCHPOINT, ELCL
                + "matchpoint.gif"); //$NON-NLS-1$		   

    }

    /**
     * Declare an Image in the registry table.
     * 
     * @param key
     *            The key to use when registering the image
     * @param path
     *            The path where the image can be found. This path is relative
     *            to where this plugin class is found (i.e. typically the
     *            packages directory)
     */
    private final static void declareRegistryImage(String key, String path) {
        ImageDescriptor desc = ImageDescriptor.getMissingImageDescriptor();
        try {
            desc = ImageDescriptor.createFromURL(makeIconFileURL(path));
        } catch (MalformedURLException me) {
            logger.log(IStatus.ERROR, "Cannot create URL", me); //$NON-NLS-1$
        }
        imageRegistry.put(key, desc);
        imageDescriptors.put(key, desc);
    }

    /**
     * Returns the ImageRegistry.
     */
    public static ImageRegistry getImageRegistry() {
        if (imageRegistry == null) {
            initializeImageRegistry();
        }
        return imageRegistry;
    }

    /**
     * Initialize the image registry by declaring all of the required graphics.
     * This involves creating JFace image descriptors describing how to
     * create/find the image should it be needed. The image is not actually
     * allocated until requested.
     * 
     * Prefix conventions Wizard Banners WIZBAN_ Preference Banners PREF_BAN_
     * Property Page Banners PROPBAN_ Color toolbar CTOOL_ Enable toolbar ETOOL_
     * Disable toolbar DTOOL_ Local enabled toolbar ELCL_ Local Disable toolbar
     * DLCL_ Object large OBJL_ Object small OBJS_ View VIEW_ Product images
     * PROD_ Misc images MISC_
     * 
     * Where are the images? The images (typically gifs) are found in the same
     * location as this plugin class. This may mean the same package directory
     * as the package holding this class. The images are declared using
     * this.getClass() to ensure they are looked up via this plugin class.
     * 
     * @see org.eclipse.jface.resource.ImageRegistry
     */
    public static ImageRegistry initializeImageRegistry() {
        imageRegistry = new ImageRegistry(DDStateLaunchPlugin.getStandardDisplay());
        imageDescriptors = new HashMap(30);
        declareImages();
        return imageRegistry;
    }

    /**
     * Returns the <code>Image<code> identified by the given key,
     * or <code>null</code> if it does not exist.
     */
    public static Image getImage(String key) {
        return getImageRegistry().get(key);
    }

    /**
     * Returns the <code>ImageDescriptor<code> identified by the given key,
     * or <code>null</code> if it does not exist.
     */
    public static ImageDescriptor getImageDescriptor(String key) {
        if (imageDescriptors == null) {
            initializeImageRegistry();
        }
        return (ImageDescriptor) imageDescriptors.get(key);
    }

    private static URL makeIconFileURL(String iconPath)
            throws MalformedURLException {
        if (ICON_BASE_URL == null) {
            throw new MalformedURLException();
        }

        return new URL(ICON_BASE_URL, iconPath);
    }

}