package org.deltadebugging.ddstate.ui;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.ui.resultmanager.DDRunResultManager;
import org.deltadebugging.ddstate.ui.resultmanager.IDDRunResultManager;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class DDStateUIPlugin extends AbstractUIPlugin {
	// The shared instance.
	private static DDStateUIPlugin plugin;

	// Resource bundle.
	private ResourceBundle resourceBundle;

	// result manager for the dd run results
	IDDRunResultManager fDDRunResultManager = null;

	// use for tracing
	public final static boolean DEBUG;

	public final static String PLUGIN_ID = "org.deltadebugging.ddstate.ui"; //$NON-NLS-1$

	public final static String DDRUN_RESULT_VIEW_ID = PLUGIN_ID
			+ ".views.DDRunResultView"; //$NON-NLS-1$

	public final static String CTS_RESULT_VIEW_ID = PLUGIN_ID
			+ ".views.cts.CTSResultView"; //$NON-NLS-1$

	static {
		DEBUG = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID //$NON-NLS-1$
				+ "/debug")); //$NON-NLS-1$
	}

	/**
	 * The constructor.
	 */
	public DDStateUIPlugin() {
		super();
		plugin = this;
		try {
			this.resourceBundle = ResourceBundle
					.getBundle("org.deltadebugging.ddstate.ui.DDStateUIPluginResources"); //$NON-NLS-1$
		} catch (MissingResourceException x) {
			this.resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static DDStateUIPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = DDStateUIPlugin.getDefault()
				.getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return this.resourceBundle;
	}

	/**
	 * Returns the standard display to be used. The method first checks, if the
	 * thread calling this method has an associated display. If so, this display
	 * is returned. Otherwise the method returns the default display.
	 */
	public static Display getStandardDisplay() {
		Display display = Display.getCurrent();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	/**
	 * Return a logger for the given tracer class.
	 * 
	 * @param tracer
	 *            the tracer is the class that have to log or trace messages.
	 * @return a logger for the given tracer class.
	 */
	public static DDLogger getLogger(Class tracer) {
		return new DDLogger(DDStateUIPlugin.getDefault(), tracer);
	}

	/**
	 * Returns the dd run result manager. The manager will be created lazily on
	 * the first access.
	 * 
	 * @return IDDRunResultManager
	 */
	public IDDRunResultManager getDDRunResultManager() {
		if (this.fDDRunResultManager == null) {
			this.fDDRunResultManager = new DDRunResultManager();
		}
		return this.fDDRunResultManager;
	}

}