/*
 * $Id: IDDRunResultManager.java,v 1.1 2005/04/20 09:15:47 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 09.04.2005 17:19:19 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.resultmanager;

import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;

/**
 * COMMENTME Comment type IDDRunResultManager
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IDDRunResultManager {

    /**
     * Registers the given listener for dd run result notifications. Has no
     * effect if an identical listener is already registered.
     * 
     * @param listener
     *            listener to register
     */
    public void addDDRunResultListener(IDDRunResultListener listener);

    /**
     * Deregisters the given listener for dd run result notifications. Has no
     * effect if an identical listener is not already registered.
     * 
     * @param listener
     *            listener to deregister
     */
    public void removeDDRunResultListener(IDDRunResultListener listener);

    /**
     * Adds the given dd run results to the result manager. Has no effect for
     * equivalent dd run results already registered. The dd run results will be
     * added to any existing dd run result views.
     * 
     * @param consoles
     *            consoles to add
     */
    public void addDDRunResults(IDDRunResult[] results);

    /**
     * Removes the given dd run results from the result manager. If the results
     * are being displayed in any dd run result views, the associated pages will
     * be closed.
     * 
     * @param consoles
     *            consoles to remove
     */
    public void removeDDRunResults(IDDRunResult[] results);

    /**
     * Returns a collection of dd run results registered with the dd run result
     * manager.
     * 
     * @return a collection of dd run results registered with the dd run result
     *         manager
     */
    public IDDRunResult[] getDDRunResults();

    /**
     * Opens the dd run result view and displays the given dd run result. If the
     * view is already open, it is brought to the front unless the view is
     * pinned on a dd run result other than the given dd run result.
     * 
     * @param result
     *            dd run result to display
     */
    public void showDDRunResultView(IDDRunResult result);
}