package org.deltadebugging.ddstate.ui.views;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.ui.DDStateUIPlugin;
import org.deltadebugging.ddstate.ui.resultmanager.IDDRunResultListener;
import org.deltadebugging.ddstate.ui.resultmanager.IDDRunResultManager;
import org.deltadebugging.ddstate.ui.views.actions.DDRunDropDownAction;
import org.deltadebugging.ddstate.ui.views.actions.PinDDRunResultAction;
import org.deltadebugging.ddstate.ui.views.actions.RemoveAllDDRunResultAction;
import org.deltadebugging.ddstate.ui.views.actions.RemoveDDRunResultAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.IPageBookViewPage;
import org.eclipse.ui.part.MessagePage;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.PageBookView;

public class DDRunResultView extends PageBookView implements IDDRunResultView,
        IDDRunResultListener, IPropertyChangeListener {

    /**
     * Whether this console is pinned.
     */
    private boolean fPinned = false;

    /**
     * Stack of dd run results in MRU order
     */
    List fStack = new ArrayList();

    /**
     * The dd run result being displayed, or <code>null</code> if none
     */
    private IDDRunResult fActiveDDRunResult = null;

    /**
     * Map of dd run results to dummy dd run parts (used to close pages)
     */
    Map fDDRunResultToPart;

    /**
     * Map of parts to dd run results
     */
    Map fPartToDDRunResult;

    // actions
    private DDRunDropDownAction fDisplayDDRunResultAction = null;
    private PinDDRunResultAction fPinAction = null;
    private RemoveDDRunResultAction fRemoveAction = null;
    private RemoveAllDDRunResultAction fRemoveAllAction = null;

    /**
     * Construct a dd run result view
     */
    public DDRunResultView() {
        this.fDDRunResultToPart = new HashMap();
        this.fPartToDDRunResult = new HashMap();
    }

    boolean isAvailable() {
        return getPageBook() != null && !getPageBook().isDisposed();
    }

    public void createPartControl(Composite parent) {
        super.createPartControl(parent);

        this.createActions();
        IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();
        this.configureToolBar(tbm);
        this.updateForExistingDDRunResults();
        getViewSite().getActionBars().updateActionBars();
    }

    /**
     * @param tbm
     */
    private void configureToolBar(IToolBarManager tbm) {
        tbm.add(new Separator("removeGroup")); //$NON-NLS-1$
        tbm.add(this.fRemoveAction);
        tbm.add(this.fRemoveAllAction);
        tbm.add(new Separator("fixedGroup")); //$NON-NLS-1$
        tbm.add(this.fPinAction);
        tbm.add(this.fDisplayDDRunResultAction);
    }

    private void createActions() {
        this.fDisplayDDRunResultAction = new DDRunDropDownAction(this);
        this.fPinAction = new PinDDRunResultAction(this);
        this.fRemoveAction = new RemoveDDRunResultAction(this);
        this.fRemoveAllAction = new RemoveAllDDRunResultAction(this);
    }

    private void updateActions() {
        if (this.fPinAction != null) {
            this.fPinAction.update();
        }
    
        if (this.fRemoveAction != null) {
            this.fRemoveAction.update();
        }
        
        if (this.fRemoveAllAction != null) {
            this.fRemoveAllAction.update();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.PageBookView#createDefaultPage(org.eclipse.ui.part.PageBook)
     */
    protected IPage createDefaultPage(PageBook book) {
        MessagePage page = new MessagePage();
        page.createControl(getPageBook());
        initPage(page);
        return page;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.PageBookView#doCreatePage(org.eclipse.ui.IWorkbenchPart)
     */
    protected PageRec doCreatePage(IWorkbenchPart dummyPart) {
        DDRunWorkbenchPart part = (DDRunWorkbenchPart) dummyPart;
        IDDRunResult ddRunResult = part.getDDRunResult();
        IPageBookViewPage page = new DDRunResultPage(this, ddRunResult);
        this.initPage(page);
        page.createControl(getPageBook());

        PageRec rec = new PageRec(dummyPart, page);

        return rec;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.PageBookView#doDestroyPage(org.eclipse.ui.IWorkbenchPart,
     *      org.eclipse.ui.part.PageBookView.PageRec)
     */
    protected void doDestroyPage(IWorkbenchPart part, PageRec pageRecord) {
        IPage page = pageRecord.page;
        page.dispose();
        pageRecord.dispose();

        IDDRunResult ddRunResult = (IDDRunResult) this.fPartToDDRunResult
                .get(part);

        // Empty cross-reference cache
        this.fPartToDDRunResult.remove(part);
        this.fDDRunResultToPart.remove(ddRunResult);

        this.updateActions();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.PageBookView#getBootstrapPart()
     */
    protected IWorkbenchPart getBootstrapPart() {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.PageBookView#isImportant(org.eclipse.ui.IWorkbenchPart)
     */
    protected boolean isImportant(IWorkbenchPart part) {
        return part instanceof DDRunWorkbenchPart;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultView#display(org.deltadebugging.ddstate.ui.views.IDDRunResult)
     */
    public void display(IDDRunResult result) {
        if (!this.isPinned()) {
            DDRunWorkbenchPart part = (DDRunWorkbenchPart) this.fDDRunResultToPart
                    .get(result);

            if (part != null) {
                partActivated(part);
            }
        }
    }

    protected void updateTitle() {
        IDDRunResult ddRunResult = this.getDDRunResult();
        if (ddRunResult == null) {
            this.setContentDescription("No DD Run Result available!"); //$NON-NLS-1$
        } else {
            this.setContentDescription(ddRunResult.getName());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultView#pin(org.deltadebugging.ddstate.ui.views.IDDRunResult)
     */
    public void pin(IDDRunResult result) {
        if (result == null) {
            this.fPinned = false;
        } else {
            this.fPinned = false; // need this off to change displayed result
            display(result);
            this.fPinned = true;
        }
        if (this.fPinAction != null) {
            this.fPinAction.update();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultView#isPinned()
     */
    public boolean isPinned() {
        return this.fPinned;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultView#getDDRunResult()
     */
    public IDDRunResult getDDRunResult() {
        return this.fActiveDDRunResult;
    }

    /**
     * Returns a stack of dd run results in the view in MRU order.
     * 
     * @return a stack of dd run results in the view in MRU order.
     * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
     */
    public List getDDRunResultStack() {
        return this.fStack;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultListener#ddRunResultsAdded(org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult[])
     */
    public void ddRunResultsAdded(final IDDRunResult[] results) {
        if (this.isAvailable()) {
            Runnable r = new Runnable() {
                public void run() {
                    for (int i = 0; i < results.length; i++) {
                        if (isAvailable()) {
                            IDDRunResult result = results[i];
                            DDRunWorkbenchPart part = new DDRunWorkbenchPart(
                                    result, getSite());
                            DDRunResultView.this.fDDRunResultToPart.put(result,
                                    part);
                            DDRunResultView.this.fPartToDDRunResult.put(part,
                                    result);
                            partActivated(part);
                        }
                    }
                }
            };
            asyncExec(r);

        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultListener#ddRunResultsRemoved(org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult[])
     */
    public void ddRunResultsRemoved(final IDDRunResult[] results) {
        if (isAvailable()) {
            Runnable r = new Runnable() {
                public void run() {
                    for (int i = 0; i < results.length; i++) {
                        if (isAvailable()) {
                            IDDRunResult ddRunResult = results[i];
                            DDRunResultView.this.fStack.remove(ddRunResult);
                            DDRunWorkbenchPart part = (DDRunWorkbenchPart) DDRunResultView.this.fDDRunResultToPart
                                    .get(ddRunResult);

                            if (part != null) {
                                partClosed(part);
                            }

                            if (getDDRunResult() == null) {
                                IDDRunResult[] available = DDStateUIPlugin
                                        .getDefault().getDDRunResultManager()
                                        .getDDRunResults();
                                if (available.length > 0) {
                                    display(available[available.length - 1]);
                                }
                            }
                        }
                    }
                }
            };
            this.asyncExec(r);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent event) {
    		// do nothing
    }

    /**
     * Register the given runnable with the display associated with this view's
     * control, if any.
     * 
     * @see org.eclipse.swt.widgets.Display#asyncExec(java.lang.Runnable)
     */
    public void asyncExec(Runnable r) {
        if (isAvailable()) {
            getPageBook().getDisplay().asyncExec(r);
        }
    }

    /**
     * Initialize for existing consoles
     */
    private void updateForExistingDDRunResults() {
        IDDRunResultManager manager = DDStateUIPlugin.getDefault()
                .getDDRunResultManager();

        // create pages for dd run results
        IDDRunResult[] ddRunResults = manager.getDDRunResults();
        this.ddRunResultsAdded(ddRunResults);

        // add as a listener
        manager.addDDRunResultListener(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.PageBookView#showPageRec(org.eclipse.ui.part.PageBookView.PageRec)
     */
    protected void showPageRec(PageRec pageRec) {
        if (!isPinned()) {
            super.showPageRec(pageRec);
            this.fActiveDDRunResult = (IDDRunResult) this.fPartToDDRunResult
                    .get(pageRec.part);
            this.fStack.remove(this.fActiveDDRunResult);
            this.fStack.add(0, this.fActiveDDRunResult);
            updateTitle();

            this.updateActions();
        }
    }

    public void dispose() {
        super.dispose();
        DDStateUIPlugin.getDefault().getDDRunResultManager()
                .removeDDRunResultListener(this);
    }
}