/*
 * $Id: DeltaHistoryTableViewer.java,v 1.3 2005/08/09 13:27:21 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 19.04.2005 17:06:36 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.views;

import org.deltadebugging.ddcore.DD;
import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.launcher.ddrun.IDDTestResult;
import org.deltadebugging.ddstate.ui.images.DDStateUIImages;
import org.deltadebugging.ddstate.ui.images.IDDStateUIImageConstants;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

/**
 * COMMENTME Comment type GraphDeltaTableViewer
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DeltaHistoryTableViewer extends TableViewer {

	private IDDRunResult fDDRunResult = null;

	public DeltaHistoryTableViewer(Composite parent, int style) {
		super(parent, style);
	}

	public DeltaHistoryTableViewer(Table table) {
		super(table);
	}

	public DeltaHistoryTableViewer(Composite parent) {
		super(parent);
	}

	{
		this.setContentProvider(new ViewContentProvider());
		this.setLabelProvider(new ViewLabelProvider());
	}

	public void setDDRunResult(IDDRunResult ddRunResult) {
		this.fDDRunResult = ddRunResult;

		super.setInput(ddRunResult.getTestResults());
	}

	String getDescription(DeltaSet deltaSet) {
		String result = "";

		DeltaSet minimizedDS = this.fDDRunResult.getMinimizedDeltaSet();
		DeltaSet computedDS = this.fDDRunResult.getComputedDeltaSet();

		int dsSize = deltaSet.size();

		if (computedDS != null && dsSize == computedDS.size()) {
			result = "Computed delta set";

		} else if (dsSize == 0) {
			result = "Empty delta set";

		} else if (dsSize == 1) {
			result = "1 delta";

		} else {
			result = dsSize + " deltas";

		}

		if (minimizedDS != null && dsSize == minimizedDS.size()
				&& deltaSet.containsAll(minimizedDS)) {
			result = "Minimized delta set";
		}

		return result;
	}

	class ViewContentProvider implements IStructuredContentProvider {
		IDDTestResult[] testResults = new IDDTestResult[0];

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			if (newInput instanceof IDDTestResult[]) {
				this.testResults = (IDDTestResult[]) newInput;
			}

		}

		public Object[] getElements(Object parent) {
			return this.testResults;
		}

		public void dispose() {
			this.testResults = null;
		}
	}

	class ViewLabelProvider extends LabelProvider implements
			ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			String result = ""; //$NON-NLS-1$

			IDDTestResult testResult = (IDDTestResult) obj;

			switch (index) {
			case 0:
				result = getText("" + testResult.getSerialNumber() + ". ");
				break;
			case 1:
				result = getText(getDescription(testResult.getTestedDeltas()));
				break;

			default:
				break;
			}

			return result;
		}

		public Image getColumnImage(Object obj, int index) {

			Image result = null;

			if (index == 2) {

				IDDTestResult testResult = (IDDTestResult) obj;

				String key = null;
				switch (testResult.getOutcome()) {
				case DD.PASS:
					key = IDDStateUIImageConstants.IMG_OUTCOME_PASS;
					break;

				case DD.FAIL:
					key = IDDStateUIImageConstants.IMG_OUTCOME_FAIL;
					break;

				case DD.UNRESOLVED:
					key = IDDStateUIImageConstants.IMG_OUTCOME_UNRESOLVED;
					break;

				}

				result = DDStateUIImages.getImage(key);
			}

			return result;
		}
	}

}