package org.deltadebugging.ddstate.visualizer;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.deltadebugging.ddstate.logger.DDLogger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class VisualizerPlugin extends AbstractUIPlugin {
    //The shared instance.
    private static VisualizerPlugin plugin;

    //Resource bundle.
    private ResourceBundle resourceBundle;

    // use this variables for tracing
    public final static boolean DEBUG;
    public final static boolean DEBUG_GRAPH_VIUSALIZER;
    public final static boolean DEBUG_GRAPH_DIFF_VISUALIZER;
    public final static boolean DEBUG_GRAPH_DIFF_VISUALIZER_SHOW_FLAGS;

    public final static String PLUGIN_ID = "org.deltadebugging.ddstate.visualizer";

    static {
        DEBUG = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID
                + "/debug"));
        DEBUG_GRAPH_VIUSALIZER = "true".equalsIgnoreCase(Platform
                .getDebugOption(PLUGIN_ID + "/debug/GraphVisualizer"));
        DEBUG_GRAPH_DIFF_VISUALIZER = "true".equalsIgnoreCase(Platform
                .getDebugOption(PLUGIN_ID + "/debug/GraphDiffVisualizer"));
        DEBUG_GRAPH_DIFF_VISUALIZER_SHOW_FLAGS = "true".equalsIgnoreCase(Platform
                .getDebugOption(PLUGIN_ID
                        + "/debug/GraphDiffVisualizer/showFlags"));
    }

    /**
     * The constructor.
     */
    public VisualizerPlugin() {
        super();
        plugin = this;
        try {
            this.resourceBundle = ResourceBundle
                    .getBundle("org.deltadebugging.ddstate.visualizer.VisualizerPluginResources");
        } catch (MissingResourceException x) {
            this.resourceBundle = null;
        }
    }

    /**
     * This method is called upon plug-in activation
     */
    public void start(BundleContext context) throws Exception {
        super.start(context);
    }

    /**
     * This method is called when the plug-in is stopped
     */
    public void stop(BundleContext context) throws Exception {
        super.stop(context);
    }

    /**
     * Returns the shared instance.
     */
    public static VisualizerPlugin getDefault() {
        return plugin;
    }

    /**
     * Returns the string from the plugin's resource bundle, or 'key' if not
     * found.
     */
    public static String getResourceString(String key) {
        ResourceBundle bundle = VisualizerPlugin.getDefault()
                .getResourceBundle();
        try {
            return (bundle != null) ? bundle.getString(key) : key;
        } catch (MissingResourceException e) {
            return key;
        }
    }

    /**
     * Returns the plugin's resource bundle,
     */
    public ResourceBundle getResourceBundle() {
        return this.resourceBundle;
    }

    /**
     * Opens an error dialog in a synchronized thread to display the given
     * error. Use this method if the error object being displayed does not
     * contain child items, or if you wish to display all such items without
     * filtering.
     * 
     * @param dialogTitle
     *            the title to use for the dialog, or <code>null</code> to
     *            indicate that the default title should be used
     * @param message
     *            the message to show in the dialog, or <code>null</code> to
     *            indicate that the error's message should be shown as the
     *            primary message
     * @param status
     *            the error to show to the user
     */
    public static void openErrorDialog(final String dialogTitle,
            final String message, final IStatus status) {
        Display.getDefault().asyncExec(new Runnable() {
            public void run() {
                ErrorDialog.openError(VisualizerPlugin
                        .getActiveWorkbenchShell(), dialogTitle, message,
                        status);
            }

        });
    }

    /**
     * Returns the active workbench shell.
     * 
     * @return the active workbench shell.
     */
    public static Shell getActiveWorkbenchShell() {
        IWorkbenchWindow workBenchWindow = getActiveWorkbenchWindow();
        if (workBenchWindow == null)
            return null;
        return workBenchWindow.getShell();
    }

    /**
     * Returns the active workbench window.
     * 
     * @return the active workbench window.
     */
    public static IWorkbenchWindow getActiveWorkbenchWindow() {
        if (plugin == null)
            return null;
        IWorkbench workBench = plugin.getWorkbench();
        if (workBench == null)
            return null;
        return workBench.getActiveWorkbenchWindow();
    }

    /**
     * Return a logger for the given tracer class.
     * 
     * @param tracer
     *            the tracer is the class that have to log or trace messages.
     * @return a logger for the given tracer class.
     */
    public static DDLogger getLogger(Class tracer) {
        return new DDLogger(VisualizerPlugin.getDefault(), tracer);
    }
}