/*
 * $Id: VizRootNode.java,v 1.5 2005/11/17 12:32:15 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 16.12.2004 09:50:36 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph;

import java.util.HashMap;
import java.util.Map;

import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.RootNode;
import org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttribute;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;

/**
 * The decorator class for the <code>IRootNode</code>.
 * 
 * @see org.deltadebugging.ddstate.memorygraph.IRootNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class VizRootNode extends VizGraphNode {

	private Map alreadyVisit = null;

	public VizRootNode(IRootNode rootNode) {
		super(rootNode, null);

		this.alreadyVisit = new HashMap();
		this.rootNode = this;

		// we have to register the root nodes
		this.registerCreatedVizGraphElements(this);
	}

	public VizRootNode(IRootNode rootNode, VizRootNode vizRootNode) {
		super(rootNode, vizRootNode);

		this.alreadyVisit = new HashMap();

		// we have to register the root nodes
		this.registerCreatedVizGraphElements(this);
	}

	/**
	 * @see org.deltadebugging.ddstate.memorygraph.IRootNode#isPassingGraph()
	 */
	public boolean isPassingGraph() {
		return this.getGraphElement().getRootNode().isPassingGraph();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizName(boolean)
	 */
	public String getGraphVizName(boolean fullQualified) {
		return RootNode.ROOT_NODE_NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizAttributes()
	 */
	public GraphVizAttributeList getGraphVizAttributes() {

		GraphVizAttributeList attrList = super.getGraphVizAttributes();
		
        if (AbstractGraphVisualizer.THESIS) {
            attrList.addAttribute(GraphVizAttribute.COLOR_PASTELRED);
        } else  {
            attrList.addAttribute(GraphVizAttribute.COLOR_RED);
        }

		return attrList;
	}

	/**
	 * Register the given graph element. The registration is usefull, because we
	 * don't want to create a viz graph element for a given graph node twice.
	 * 
	 * @param graphElement
	 *            the graph element you want to register.
	 */
	public void registerCreatedVizGraphElements(VizGraphElement graphElement) {
		this.alreadyVisit.put(graphElement.getGraphElement(), graphElement);
	}

	/**
	 * Look if for the given graph element a corresponding viz graph element
	 * exists.
	 * 
	 * @param graphElement
	 *            the graph element you want to check for a corresponding viz
	 *            graph element.
	 * @return the viz graph element for the graph element if one exists,
	 *         otherwise <code>null</code>.
	 */
	public VizGraphElement alreadyRegister(IGraphElement graphElement) {
		return (VizGraphElement) this.alreadyVisit.get(graphElement);
	}

}
