/*
 * $Id: GraphExtractorSemaphore.java,v 1.5 2005/08/28 11:32:49 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 07.02.2005 11:50:24 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.extractor;

import org.deltadebugging.ddstate.extractor.exceptions.WaitTimeExpiredException;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * COMMENTME Comment type GraphExtractorSemaphore
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphExtractorSemaphore {

    private final static int MAX_WAIT_TIME = 6000000; // in milli seconds
    private final static int CHECK_TIME = 1000; // in milli seconds

    private IRootNode fPassGraph = null;
    private IRootNode fFailGraph = null;
    private boolean fPassingGraphExtracted = false;
    private boolean fFailingGraphExtracted = false;

    public synchronized void waitForGraphs(IProgressMonitor monitor)
            throws WaitTimeExpiredException {
        int counter = MAX_WAIT_TIME / CHECK_TIME;

        while ((!this.isPassGraphExtracted() || !this.isFailGraphExtracted())
                && !monitor.isCanceled() && counter > 0) {
            try {
                this.wait(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } finally {
                counter--;

                // Cancel the run if the both graphs are not extracted in the
                // MAX_WAIT_TIME
                if (counter == 0)
                    throw new WaitTimeExpiredException("The wait time of " + MAX_WAIT_TIME + "ms is expired");
            }
        }
    }

    public synchronized void passGraphExtracted(IRootNode passGraph) {
        this.fPassingGraphExtracted = true;
    	this.fPassGraph = passGraph;
        this.notify();
    }

    public synchronized void failGraphExtracted(IRootNode failGraph) {
    	this.fFailingGraphExtracted = true;
    	this.fFailGraph = failGraph;
        this.notify();
    }

    public boolean isPassGraphExtracted() {
        return this.fPassingGraphExtracted;
    }

    public boolean isFailGraphExtracted() {
        return this.fFailingGraphExtracted;
    }

    public IRootNode getPassGraph() {
        return this.fPassGraph;
    }

    public IRootNode getFailGraph() {
        return this.fFailGraph;
    }
}