/*
 * $Id: StackFrame.java,v 1.3 2005/08/21 13:14:11 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 2, 2005 3:19:44 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaStackFrame;

/**
 * A stack frame in a thread on a Java virtual machine.
 * 
 * @see org.eclipse.jdt.debug.core.IJavaStackFrame
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class StackFrame implements IStackFrame {

	private final IJavaStackFrame fStackFrame;

	private final IMethodIdentifier fMethodIndentifier;

	private final int fLineNumber;

	private final int fFrameNumber;

	private final int fFramesCount;

	public StackFrame(IJavaStackFrame stackFrame, int frameNumber,
			int framesCount) throws MemoryGraphExtractionException {
		this.fStackFrame = stackFrame;
		this.fFrameNumber = frameNumber;
		this.fFramesCount = framesCount;

		try {
			this.fLineNumber = stackFrame.getLineNumber();
			this.fMethodIndentifier = new MethodIdentifier(stackFrame
					.getDeclaringTypeName(), stackFrame.getMethodName(),
					stackFrame.getSignature());
		} catch (DebugException e) {
			throw new MemoryGraphExtractionException(
					"Cannot retrieve necessary informations from the given stackframe",
					e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IStackFrame#getFrameNumber()
	 */
	public int getFrameNumber() {
		return this.fFrameNumber;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IStackFrame#getFramesCount()
	 */
	public int getFramesCount() {
		return this.fFramesCount;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IStackFrame#getLineNumber()
	 */
	public int getLineNumber() {
		return this.fLineNumber;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IStackFrame#getUnderlyingStackFrame()
	 */
	public IJavaStackFrame getUnderlyingStackFrame() {
		return this.fStackFrame;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();

		result.append(this.fFrameNumber + "/" + this.fFramesCount + ". ");
		result.append(this.fMethodIndentifier.toString());

		String lineNumber = "n.a. [native method]";
		if (this.fLineNumber != -1) {
			lineNumber = "" + this.fLineNumber;
		}

		result.append(" line: " + lineNumber);

		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IStackFrame#getMethodIdentifier()
	 */
	public IMethodIdentifier getMethodIdentifier() {
		return this.fMethodIndentifier;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		return this.toString().equals(obj.toString());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return this.toString().hashCode();
	}
}
