/*
 * $Id: CreateObjectDelta.java,v 1.11 2005/11/22 14:23:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 29.07.2005 18:52:53 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.create;

import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.launcher.client.ObjectCreationException;
import org.deltadebugging.ddstate.launcher.client.TestRunnerException;
import org.deltadebugging.ddstate.memorygraph.delta.ICreateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.location.NullLocation;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.deltadebugging.ddstate.memorygraph.value.IObjectNode;
import org.eclipse.jdt.debug.core.IJavaThread;

/**
 * This delta creates a new object (simple object, array or string) if it will
 * applied.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class CreateObjectDelta extends CreateDelta implements ICreateDelta {

	/**
	 * @param objRef
	 *            the object reference to the object you want to create.
	 */
	public CreateObjectDelta(IObjectNode objRef) {
		super(new NullLocation(), objRef);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#apply(org.eclipse.jdt.debug.core.IJavaThread,
	 *      org.deltadebugging.ddstate.launcher.client.ITestRunner)
	 */
	public void apply(IJavaThread thread, ITestRunner testRunner)
			throws DeltaExecutionException {
		IObjectNode objRefNewValue = (IObjectNode) this
				.getValueToCreate();

		try {
			testRunner.createNewObject(objRefNewValue, thread);

		} catch (ObjectCreationException e) {
			throw new DeltaExecutionException("Cannot apply delta.", e);
		} catch (TestRunnerException e) {
			throw new DeltaExecutionException("Cannot apply delta.", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#ctsEqual(org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta)
	 */
	public boolean ctsEqual(IMemoryGraphDelta delta) {

		if (!(delta instanceof CreateObjectDelta)) {
			return false;
		}

		CreateObjectDelta createObjDelta = (CreateObjectDelta) delta;

		return this.getLocationToValueToCreate().equals(
				createObjDelta.getLocationToValueToCreate());
	}
}
