/*
 * $Id: DeleteObjectReferenceDelta.java,v 1.11 2005/11/22 14:23:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 29.07.2005 18:25:12 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.delete;

import org.deltadebugging.ddstate.extractor.MemoryAccessUtil;
import org.deltadebugging.ddstate.extractor.exceptions.MemoryAccessException;
import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.delta.IDeleteDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.debug.core.IJavaValue;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * This delete delete a given object reference. That means it set the given
 * object reference to <code>null</code>.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DeleteObjectReferenceDelta extends DeleteDelta implements
        IDeleteDelta {

    /**
     * @param objRefToDelete
     *            the objecte reference to delete.
     */
    public DeleteObjectReferenceDelta(IGraphEdge objRefToDelete) {
        super(objRefToDelete, true);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#apply(org.eclipse.jdt.debug.core.IJavaThread,
     *      org.deltadebugging.ddstate.launcher.client.ITestRunner)
     */
    public void apply(IJavaThread thread, ITestRunner testRunner)
            throws DeltaExecutionException {

        try {
            //IJavaVariable variable = this.getTmpVariableToLocation();
            IJavaVariable variable = this.getLocation().getVariableToLocation(thread, testRunner);
        	IJavaValue newValue = MemoryAccessUtil
                    .generateDefaultValueForVariable(thread, variable);
            variable.setValue(newValue);

        } catch (DebugException e) {
            throw new DeltaExecutionException("Cannot apply delta", e);
        } catch (MemoryAccessException e) {
            throw new DeltaExecutionException("Cannot apply delta", e);
        } catch (Throwable e) {
            throw new DeltaExecutionException("Cannot apply delta", e);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#ctsEqual(org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta)
     */
    public boolean ctsEqual(IMemoryGraphDelta delta) {

        if (!(delta instanceof DeleteObjectReferenceDelta)) {
            return false;
        }

        return this.getLocation().equals(delta.getLocation());
    }

}
