/*
 * $Id: PathLocation.java,v 1.10 2005/11/14 10:59:58 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University Prof. Dr. Andreas Zeller
 * 
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Common Public License v1.0 which accompanies
 * this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Created on 13.07.2005 14:32:43 by @author Karsten Lehmann Mail to:
 * lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.location;

import org.deltadebugging.ddstate.extractor.MemoryAccessUtil;
import org.deltadebugging.ddstate.extractor.exceptions.MemoryAccessException;
import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.launcher.client.TestRunnerException;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.deltadebugging.ddstate.memorygraph.util.MemoryGraphUtil;
import org.deltadebugging.ddstate.memorygraph.util.Path;
import org.deltadebugging.ddstate.memorygraph.util.PathComparator;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * COMMENTME Comment type PathLocation
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class PathLocation implements IDeltaLocation {

	private final Path fPath;

	public PathLocation(Path path) {
		this.fPath = path;
	}

	public PathLocation(IGraphElement element, boolean associatedNodes) {
		this.fPath = MemoryGraphUtil.searchPathToRoot(element, associatedNodes);
	}

	/**
	 * Returns the underlying path to this location.
	 * 
	 * @return the underlying path to this location.
	 */
	public Path getPath() {
		return this.fPath;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.location.IDeltaLocation#getVariableToLocation(org.eclipse.jdt.debug.core.IJavaThread,
	 *      org.deltadebugging.ddstate.launcher.client.ITestRunner)
	 */
	public IJavaVariable getVariableToLocation(IJavaThread thread,
			ITestRunner testRunner) throws DeltaExecutionException {
		IJavaVariable result = null;

		int classLoaderID = -1;
		try {
			classLoaderID = testRunner.getCurrentClassLoaderID();
		} catch (TestRunnerException e) {
			throw new DeltaExecutionException(
					"Cannot retrieve the id of the current class loader "
							+ "of the junit test runner.", e);
		}

		try {
			result = MemoryAccessUtil.getVariable(thread, this.fPath,
					classLoaderID);
		} catch (MemoryAccessException e) {
			throw new DeltaExecutionException(
					"Cannot find the variable to the path:\n" + this.fPath, e);
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.location.IDeltaLocation#getLocationAsString()
	 */
	public String getLocationAsString() {
		String result = null;

		IGraphEdge lastEdge = this.fPath.getLastPathElement();
		IGraphNode graphNode = lastEdge.getSourceNode();

		result = lastEdge.getName();

		if (this.fPath.size() > 1) {
			IGraphEdge edgeBeforeLastEdge = this.fPath
					.get(this.fPath.size() - 2);
			result = edgeBeforeLastEdge.getName() + "." + result;
		}

		if (graphNode instanceof IValueNode) {
			result += " (class: "
					+ ((IValueNode) graphNode).getShortReferenceTypeName()
					+ ")";
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {

		if (!(obj instanceof PathLocation)) {
			return false;
		}

		PathLocation pathLocObj = (PathLocation) obj;

		return PathComparator.getInstance().compare(this.getPath(), pathLocObj.getPath()) == 0;
	}
    
    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {        
        return this.getPath().toString();
    }

}
