/*
 * $Id: UpdateObjectValueDelta.java,v 1.6 2005/11/22 14:23:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 01.08.2005 09:15:36 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.update;

import org.deltadebugging.ddstate.memorygraph.delta.IUpdateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.MemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.location.PathLocation;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;

/**
 * This delta update the value of an object in the memory.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class UpdateObjectValueDelta extends MemoryGraphDelta implements
        IUpdateDelta {

    /** The old value of the object you want to update */
    private final IValueNode fOldValue;

    /** The new value of the object you want to update */
    private final IValueNode fNewValue;

    public UpdateObjectValueDelta(IValueNode oldValue,
            IValueNode newValue) {
        super(new PathLocation(oldValue, true));

        this.fOldValue = oldValue;
        this.fNewValue = newValue;
    }

    /**
     * Returns the old value to update.
     * 
     * @return the old value to update.
     */
    public IValueNode getOldValue() {
        return this.fOldValue;
    }

    /**
     * Returns the new value.
     * 
     * @return the new value.
     */
    public IValueNode getNewValue() {
        return this.fNewValue;
    }

}
