/*
 * $Id: UpdateStringDelta.java,v 1.7 2005/11/22 14:23:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 01.08.2005 09:19:16 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.update;

import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IUpdateDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.deltadebugging.ddstate.memorygraph.value.IStringObjectNode;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * This delta updates an string in the memory.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class UpdateStringDelta extends UpdateObjectValueDelta implements
		IUpdateDelta {

	public UpdateStringDelta(IStringObjectNode oldValue,
			IStringObjectNode newValue) {
		super(oldValue, newValue);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#apply(org.eclipse.jdt.debug.core.IJavaThread,
	 *      org.deltadebugging.ddstate.launcher.client.ITestRunner)
	 */
	public void apply(IJavaThread thread, ITestRunner testRunner)
			throws DeltaExecutionException {
		try {
			IJavaVariable variable = this.getLocation().getVariableToLocation(
					thread, testRunner);
			variable.setValue(((IStringObjectNode) this.getNewValue())
					.getStringValue());

		} catch (DebugException e) {
			throw new DeltaExecutionException("Cannot apply delta", e);
		} catch (Throwable e) {
			throw new DeltaExecutionException("Cannot apply delta", e);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#ctsEqual(org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta)
	 */
	public boolean ctsEqual(IMemoryGraphDelta delta) {

		if (!(delta instanceof UpdateStringDelta)) {
			return false;
		}

		return this.getLocation().equals(delta.getLocation());
	}

}
