/*
 * $Id: GraphFilterManager.java,v 1.3 2005/03/24 08:05:32 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Sep 17, 2004 11:47:10 AM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.filter;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

/**
 * COMMENTME Comment type GraphFilterManager
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphFilterManager implements IGraphFilter {

    private Map filters = null;

    /**
     * Constructor for the GraphFilterManager.
     *  
     */
    public GraphFilterManager() {
        this.filters = new HashMap();
    }

    /**
     * Add the given filter to the set of filters if the filter is not
     * <code>null</code> and the filter is not contained already.
     * 
     * @param filter
     *            the filter to add.
     */
    public void addFilter(IGraphFilter filter) {
        if (filter == null)
            return;

        if (!this.filters.containsKey(filter)) {
            this.filters.put(filter, filter);
        }
    }

    /**
     * Add all given filters to the set of filters if the given vector is not
     * <code>null</code>.
     * 
     * @param filters
     */
    public void addAllFilter(Vector filtersToAdd) {
        if (filtersToAdd == null)
            return;

        for (Iterator itFilters = filtersToAdd.iterator(); itFilters.hasNext();) {
            Object filter = itFilters.next();

            // check if the element is of the required type
            if (filter instanceof IGraphFilter) {
                this.addFilter((IGraphFilter) filter);
            }
        }

    }

    /**
     * Remove the given filter of the set of filters.
     * 
     * @param filter
     *            the filter to remove.
     * @throws NullPointerException
     *             if the given filter is null.
     */
    public void removeFilter(IGraphFilter filter) {
        if (filter == null)
            throw new NullPointerException("Given filter must not be null.");

        if (this.filters.containsKey(filter)) {
            this.filters.remove(filter);
        }

    }

    /**
     * Remove all filters from the manager.
     *  
     */
    public void removeAllFilter() {
        if (!this.filters.isEmpty()) {
            this.filters = new HashMap();
        }
    }

    /**
     * Call <code>filterObject</code> of all filters. If one filter return
     * true this method return always true, else false.
     * 
     * @return true if one filter return true for the given object.
     * @throws GraphFilterException
     *             if something went wrong if the filter was applied.
     * @throws NullPointerException
     *             if the given graph node is null.
     */
    public boolean filterObject(Object obj) throws GraphFilterException {
        if (obj == null)
            throw new NullPointerException("Given object must not be null.");

        boolean result = false;

        for (Iterator itFilters = this.filters.values().iterator(); itFilters
                .hasNext();) {
            IGraphFilter filter = (IGraphFilter) itFilters.next();

            if (filter.filterObject(obj)) {
                result = true;
                break;
            }
        }

        return result;
    }
}