/*
 * $Id: IArrayElementEdge.java,v 1.1 2005/08/03 13:16:51 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.08.2005 14:16:52 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;

/**
 * This edge joined an array reference with one of his element. In addition to
 * ordinary graph edges this edge provides a method to get the index of the
 * array element. Also a method for retrieving the array length is implemented
 * too.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IArrayElementEdge extends IGraphEdge {

    /**
     * Returns the index of the array element (target node) in the array
     * reference (source node).
     * 
     * @return the index of the array element.
     */
    public abstract int getArrayIndex();

    /**
     * Returns the length of the array reference (source node).
     * 
     * @return the length of the array reference.
     */
    public abstract int getArrayLenght();
}